/*
 *  The Mana Client
 *  Copyright (C) 2011-2012  The Mana Developers
 *
 *  This file is part of The Mana Client.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once

#include "gui/widgets/window.h"

#include <guichan/actionlistener.hpp>
#include <guichan/keylistener.hpp>
#include <guichan/listmodel.hpp>

#include <memory>

class Button;
class DropDown;
class ServerDialog;
class TextField;

/**
 * Server Type List Model
 */
class TypeListModel : public gcn::ListModel
{
    public:
        TypeListModel() {}

        /**
         * Used to get number of line in the list
         */
        int getNumberOfElements() override { return 2; }

        /**
         * Used to get an element from the list
         */
        std::string getElementAt(int elementIndex) override;
};

/**
 * The custom server addition dialog.
 *
 * \ingroup Interface
 */
class CustomServerDialog : public Window,
                     public gcn::ActionListener,
                     public gcn::KeyListener
{
    public:
        CustomServerDialog(ServerDialog *parent, int index = -1);
        ~CustomServerDialog() override;

        /**
         * Called when receiving actions from the widgets.
         */
        void action(const gcn::ActionEvent &event) override;

        void keyPressed(gcn::KeyEvent &keyEvent) override;

    private:
        TextField *mServerAddressField;
        TextField *mPortField;
        TextField *mNameField;
        TextField *mDescriptionField;
        Button *mOkButton;
        Button *mCancelButton;
#ifdef MANASERV_SUPPORT
        DropDown *mTypeField;
        std::unique_ptr<TypeListModel> mTypeListModel;
#endif
        ServerDialog *mServerDialog;
        // The index of the entry to modify, -1 when only adding a new entry.
        int mIndex;
};
