/*
 *  The Mana Client
 *  Copyright (C) 2004-2009  The Mana World Development Team
 *  Copyright (C) 2009-2012  The Mana Developers
 *
 *  This file is part of The Mana Client.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once

namespace gcn
{
    class ActionEvent;
    class ActionListener;
    class AllegroFont;
    class AllegroGraphics;
    class AllegroImage;
    class AllegroImageLoader;
    class AllegroInput;
    class BasicContainer;
    class Button;
    class CheckBox;
    class ClipRectangle;
    class Color;
    class Container;
    class DefaultFont;
    class DropDown;
    class Event;
    class Exception;
    class FocusHandler;
    class FocusListener;
    class Font;
    class GenericInput;
    class Graphics;
    class Gui;
    class HGEGraphics;
    class HGEImage;
    class HGEImageFont;
    class HGEImageLoader;
    class HGEInput;
    class Icon;
    class Image;
    class ImageButton;
    class InputEvent;
    class ImageFont;
    class ImageLoader;
    class Input;
    class Key;
    class KeyEvent;
    class KeyInput;
    class KeyListener;
    class Label;
    class ListBox;
    class ListModel;
    class MouseEvent;
    class MouseInput;
    class MouseListener;
    class OpenGLAllegroImageLoader;
    class OpenGLGraphics;
    class OpenGLImage;
    class OpenGLSDLImageLoader;
    class OpenLayerFont;
    class OpenLayerGraphics;
    class OpenLayerImage;
    class OpenLayerImageLoader;
    class OpenLayerInput;
    class RadioButton;
    class Rectangle;
    class ScrollArea;
    class SDLGraphics;
    class SDLImage;
    class SDLImageLoader;
    class SDLInput;
    class SDLPixel;
    class SelectionEvent;
    class SelectionListener;
    class Slider;
    class Tab;
    class TabbedArea;
    class TextBox;
    class TextField;
    class Widget;
    class WidgetListener;
    class Window;
}
