int
inflateMemory(unsigned char *in, unsigned int inLength,
              unsigned char *&out, unsigned int &outLength);

int
inflateMemory(unsigned char *in, unsigned int inLength,
              unsigned char *&out);

int
compressMemory(unsigned char *in, unsigned int inLength,
              unsigned char *&out, unsigned int &outLength);
