/*
 *  The Mana Client
 *  Copyright (C) 2004-2009  The Mana World Development Team
 *  Copyright (C) 2009-2012  The Mana Developers
 *
 *  This file is part of The Mana Client.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once

#include "gui/widgets/window.h"

#include "net/charhandler.h"

#include <guichan/actionlistener.hpp>
#include <guichan/keylistener.hpp>

class CharacterDisplay;
class LocalPlayer;
class LoginData;
class PlayerBox;

namespace Net {
class CharHandler;
}

/**
 * Character selection dialog.
 *
 * \ingroup Interface
 */
class CharSelectDialog : public Window, public gcn::ActionListener,
                         public gcn::KeyListener
{
    public:
        friend class CharDeleteConfirm;
        friend class Net::CharHandler;

        CharSelectDialog(LoginData *loginData);

        ~CharSelectDialog() override;

        void action(const gcn::ActionEvent &event) override;

        void keyPressed(gcn::KeyEvent &keyEvent) override;

        enum SelectAction {
            Focus,
            Choose
        };

        /**
         * Attempt to select the character with the given name. Returns whether
         * a character with the given name was found.
         *
         * \param action determines what to do when a character with the given
         *               name was found (just focus or also try to choose this
         *               character).
         */
        bool selectByName(const std::string &name,
                          SelectAction action = Focus);

    private:
        void attemptCharacterDelete(int index);
        void attemptCharacterSelect(int index);

        void setCharacters(const Net::Characters &characters);

        void lock();
        void unlock();
        void setLocked(bool locked);

        bool mLocked = false;

        gcn::Label *mAccountNameLabel;

        gcn::Button *mSwitchLoginButton;
        gcn::Button *mChangePasswordButton;
        gcn::Button *mUnregisterButton = nullptr;
        gcn::Button *mChangeEmailButton = nullptr;

        /** The player boxes */
        std::vector<CharacterDisplay*> mCharacterEntries;

        LoginData *mLoginData;

        Net::CharHandler *mCharHandler;
};
